/*	File:	map.c
	Author:	Alex Fedosov
	Date:	February 5, 2001
	Description:	map functions, take 2
*/

#include <stdio.h>
#include <stdlib.h>

#include "common/types.h"
#include "common/const.h"
#include "common/errors.h"
#include "common/map.h"

#include "dmzcore.h"

#define FREE(p, s) if (p != NULL) { dmz_free(p, s); p = NULL; }

/* Global Variables */

/* initial map of the world */
WORD16 dmz_world_map[MAPWIDTH][MAPHEIGHT];

/* map cell array */
HMAPCELL dmz_map_cells[MAX_MAP_CELLS];

unsigned int map_width = 0;
unsigned int map_height = 0;

void dmz_map_init(void)
{
	int i, j;

	for (i = 0; i < MAXCELLS; i++)
		dmz_map_cells[i] = NULL;

	/* initialize the map */
	for (i = 0; i < MAPWIDTH; i++)
		for (j = 0; j < MAPHEIGHT; j++)
			dmz_world_map[i][j] = 0;
}

void dmz_map_destroy(void)
{
	int i;

	for (i = 0; i < MAXCELLS; i++)
		FREE(dmz_map_cells[i], sizeof(struct dmz_map_cell));
}

int dmz_map_get_available_index(void)
{
	int i;

	for (i = 0; i < MAX_MAP_CELLS; i++)
		if (dmz_map_cells[i] == NULL)
			return i;

	return -1;
}

CELL dmz_map_new_cell(WORD16 tileid)
{
	if (dmz_map_cells[tileid] == NULL)
	{
		dmz_map_cells[tileid] = (HMAPCELL) dmz_malloc(sizeof(struct dmz_map_cell));

		if (dmz_map_cells[tileid] == NULL)
		{
			dmz_logerr("[common:map:dmz_new_map_cell] unable to allocate memory!");
			return -1;
		}
	}

	return tileid;
}

int dmz_map_set_prop(CELL cell, short *pass, short *dam, short *spd)
{
	if (dmz_map_cells[cell] == NULL)
		return DMZ_MAP_ERROR;

	if (pass != NULL)
		dmz_map_cells[cell]->passable = *pass;

	if (dam != NULL)
		dmz_map_cells[cell]->damage = *dam;

	if (spd != NULL)
		dmz_map_cells[cell]->speed = *spd;

	return DMZ_SUCCESS;
}

int dmz_map_get_prop(CELL cell, short *pass, short *dam, short *spd)
{
	if (dmz_map_cells[cell] == NULL)
		return DMZ_MAP_ERROR;

	if (pass != NULL)
		*pass = dmz_map_cells[cell]->passable;

	if (dam != NULL)
		*dam = dmz_map_cells[cell]->damage;

	if (spd != NULL)
		*spd = dmz_map_cells[cell]->speed;

	return DMZ_SUCCESS;
}
