/*	File:	move.c
	Author:	Alex Fedosov, Jeff Malch
	Date:	March 10, 2001
	Description:	movement code and collision detection
*/

#include <stdio.h>
#include <string.h>

#include "common/const.h"
#include "common/errors.h"
#include "common/map.h"
#include "common/object.h"
#include "common/msg.h"

#include "dmzcore.h"
#include "dmz_net.h"

int CellPassable(int x, int y)
{
	CELL cell;
	short pass, dam, spd;

	x = x / PIXELS_PER_CELL;
	y = y / PIXELS_PER_CELL;

	cell = dmz_world_map[x][y];

	dmz_map_get_prop(cell, &pass, &dam, &spd);

	return (int) pass;
}

/* TODO this is still not completely correct, but it works */
int ValidMove(int direction, int player)
{
	WORD32 check_x, check_y;
	WORD32 off_x, off_y;
	int result = 1, spill = 0;

	/* calculate offset into the current cell */
	off_x = dmz_objects[player]->x % PIXELS_PER_CELL;
	off_y = dmz_objects[player]->y % PIXELS_PER_CELL;

	spill = (off_x > 10) || (off_y > 10);

	check_x = dmz_objects[player]->x;
	check_y = dmz_objects[player]->y;

	switch (direction)
	{
	case NORTH:		/* check north and northeast */
		result = CellPassable(check_x, check_y);
		if (spill)
			result = result && 
				CellPassable(check_x + PIXELS_PER_CELL, check_y);
		break;
	case SOUTH:		/* check south and southeast */
		result = CellPassable(check_x, check_y + PIXELS_PER_CELL);
		if (spill)
			result = result && 
				CellPassable(check_x + PIXELS_PER_CELL, check_y + PIXELS_PER_CELL);
		break;
	case EAST:		/* check east and sotheast */
		result = CellPassable(check_x + PIXELS_PER_CELL, check_y);
		if (spill)
			result = result && 
				CellPassable(check_x + PIXELS_PER_CELL, check_y + PIXELS_PER_CELL);
		break;
	case WEST:		/* check west and southwest */
		result = CellPassable(check_x, check_y);
		if (spill)
			result = result && 
				CellPassable(check_x, check_y + PIXELS_PER_CELL);
		break;

	case NORTHEAST:
		result = CellPassable(check_x + PIXELS_PER_CELL, check_y);
		break;
	case NORTHWEST:
		result = CellPassable(check_x, check_y);
		break;
	case SOUTHEAST:
		result = CellPassable(check_x + PIXELS_PER_CELL, check_y + PIXELS_PER_CELL);
		break;
	case SOUTHWEST:
		result = CellPassable(check_x, check_y + PIXELS_PER_CELL);
		break;
	default:
		break;
	}

	return result;
}

int DetectCollision(int player)
{
	int i, other;
	int dx, dy, tx, ty;
	unsigned int cntr1_x, cntr1_y, cntr2_x, cntr2_y;

	dmz_assert(dmz_objects[player]);

	/* go through the object array */
	for (i = 1; i < MAXOBJECTS; i++)
	{
		if (dmz_objects[i] == NULL)
			continue;
		if (dmz_objects[i]->status == INVISIBLE ||
			dmz_objects[i]->status == DEAD)
			continue;
		if (dmz_objects[i]->objid == player)
			continue;

		/* valid object found, do collision detection */
		other = dmz_objects[i]->objid;

/*		sprintf(buf, "detecting collision with %d", other);
		dmz_logstd(buf);

		sprintf(buf, "player position: (%d, %d), other position: (%d, %d)", 
			dmz_objects[player]->x, dmz_objects[player]->y, dmz_objects[other]->x, dmz_objects[other]->y);
		dmz_logstd(buf);
*/
		/* find the center of both objects */
		cntr1_x = dmz_objects[player]->x + (dmz_objects[player]->width >> 1);
		cntr1_y = dmz_objects[player]->y + (dmz_objects[player]->height >> 1);

		cntr2_x = dmz_objects[other]->x + (dmz_objects[other]->width >> 1);
		cntr2_y = dmz_objects[other]->y + (dmz_objects[other]->height >> 1);

/*		sprintf(buf, "player center: (%d, %d), other center: (%d, %d)", 
			cntr1_x, cntr1_y, cntr2_x, cntr2_y);
		dmz_logstd(buf);
*/
		/* figure out the threshold width and height */
		tx = (dmz_objects[player]->width >> 1) + (dmz_objects[other]->width >> 1);
		ty = (dmz_objects[player]->height >> 1) + (dmz_objects[other]->height >> 1);

		/* figure out the distance between two centers */
		if (cntr1_x > cntr2_x)
			dx = cntr1_x - cntr2_x;
		else
			dx = cntr2_x - cntr1_x;

		if (cntr1_y > cntr2_y)
			dy = cntr1_y - cntr2_y;
		else
			dy = cntr2_y - cntr1_y;

/*		sprintf(buf, "distance: (%d, %d) threshold: (%d, %d)", dx, dy, tx, ty);
		dmz_logstd(buf);
*/
		/* if distance is smaller than threshold, we have a collision */
		if (dx < tx && dy < ty)
			return other;
	}

	return 0;
}
