/*	File:	map.h
	Author:	Alex Fedosov
	Date:	February 5, 2001
	Description:	world map interface, take 2
*/

#ifndef __DMZ_MAP_H__
#define __DMZ_MAP_H__

#include "common/types.h"
#include "common/const.h"

/* map cell */
struct dmz_map_cell
{
	WORD16 bgidx;			/* index into bitmap array giving background tile */
	short passable;			/* flag indicating whether an object can be in the cell */
	short damage;			/* damage incurred by walking on this terrain type */
	short speed;			/* speed increase or decrease */
};

typedef WORD16 CELL;
typedef struct dmz_map_cell* HMAPCELL;

/* initial map of the world -- a big array of ints for the entire map,
   plus a smaller array for different map cells. using 16-bit ints as indices
   allow us to have a map with up to 4 billion (2^32) cells. */
GLOBAL CELL dmz_world_map[MAPWIDTH][MAPHEIGHT];
GLOBAL HMAPCELL dmz_map_cells[MAX_MAP_CELLS];
GLOBAL unsigned int map_width;
GLOBAL unsigned int map_height;

/* create a new cell */
CELL dmz_map_new_cell(WORD16 tileid);

/* set the properties of a cell */
int dmz_map_set_prop(CELL cell, short *pass, short *dam, short *spd);

/* get the properties of a cell */
int dmz_map_get_prop(CELL cell, short *pass, short *dam, short *spd);

/* initialize the map */
void dmz_map_init(void);
void dmz_map_destroy(void);

#endif
