/*	File:	server.h
	Author:	Alex Fedosov
	Date:	November 14, 2000
	Description:	main server include file
*/

#ifndef __SERVER_H__
#define __SERVER_H__

#include "common/types.h"
#include "common/const.h"
#include "common/errors.h"
#include "common/map.h"
#include "common/object.h"
#include "common/msg.h"
#include "common/init.h"
#include "common/move.h"
#include "common/attack.h"
#include "common/upgrade.h"
#include "common/util.h"

#define DMZ_SERVER_VERSION 0x0106
#define MAXPLAYERS 128
#define MAXCLIENTS 128

int dmz_server_main(void *arg);
int dmz_server_init(void);
void dmz_server_kill(void);
void dump_server_stats(void);
int dmz_get_num_clients(void);
int dmz_get_num_servers(void);
int dmz_get_num_players(void);
void dmz_drop_client(int connid);
int ServerProcess(void);
void ProcessUnits(void);
int PickupItem(WORD16 objid, WORD16 item);
void UpgradeRobot(WORD16 player_id);
void dmz_player_death(WORD16 objid);
void ApplyAI(WORD16 objid);
void MoveMissile(WORD16 m);
void Regenerate(WORD16 object);

int dmz_process_msg(struct dmz_message *m, int connid);
int dmz_process_message_BEGIN(struct dmz_message *m, int connid);
int dmz_process_message_MOVE(struct dmz_message *m);
int dmz_broadcast(struct dmz_message *m);

typedef struct dmz_client
{
	int connid;
	int objid;
	char *ip_addr;
} CLIENT;

GLOBAL CLIENT clients[MAXCLIENTS];

#endif
