#ifndef __SMP_LOCK_H__
#define __SMP_LOCK_H__

typedef volatile int spin_lock_t;

#define SPIN_LOCK_INITIALIZER   0

#define spin_lock_init(s)       (*(s) = SPIN_LOCK_INITIALIZER)
#define spin_lock_locked(s)     (*(s) != SPIN_LOCK_INITIALIZER)

/* Must have gcc to compile */

#ifdef  __GNUC__

#define spin_unlock(p) \
        ({  register int _u__ ; \
            __asm__ volatile("xorl %0, %0; \n\
                          xchgl %0, %1" \
                        : "=&r" (_u__), "=m" (*(p)) ); \
            0; })

/* Orig */
#define spin_try_lock(p)\
        (!({  register int _r__; \
            __asm__ volatile("movl $1, %0; \n\
                          xchgl %0, %1" \
                        : "=&r" (_r__), "=m" (*(p)) ); \
            _r__; }))
#define spin_lock(p) \
        ({ while (!spin_try_lock(p)) while (*(p)); })

/* This is from Linux pthreads */
/*
#define spin_try_lock(p)\
        (!({  register int _r__; \
            __asm__ volatile( \
		"xchgl %0, %1" \
	       : "=r"(_r__), "=m"(*(p)) \
	       : "0"(1), "m"(*(p)) \
	       : "memory"); \
            _r__; }))
	    

#ifndef PT_EI
#define PT_EI extern inline
#endif

PT_EI int
testandset (int *spinlock)
{
  int ret;

  __asm__ __volatile__(
       "xchgl %0, %1"
       : "=r"(ret), "=m"(*spinlock)
       : "0"(1), "m"(*spinlock)
       : "memory");

  return ret;
}

#define spin_lock(p) \
        ({ while (spin_try_lock(p)); })
*/
	
#endif  /* __GNUC__ */

/*  lock datatype and actions  */
#define smp_lock_t spin_lock_t

#define smp_lock_init(a)    a = 0
#define smp_lock_free(a)    a = 0
#define smp_lock_acquire(a) spin_lock(&(a))
#define smp_lock_release(a) spin_unlock(&(a))

#endif
