/*	File:	wincli.c
	Author:	Alex Fedosov
	Date:	February 1, 2001
	Description:	Windows code to create client app
*/

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "client.h"

#include "dmz_net.h"
#include "dmzcore.h"
#include "dmzgraph.h"
#include "dmz_snd.h"
#include "resource.h"

LRESULT CALLBACK WndProc(HWND hwnd, unsigned umsg, WPARAM wparam, LPARAM lparam);
BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
void GDIBlit(HWND hWnd, HBITMAP hBit);

/* windows-related stuff */
HINSTANCE hInstance;
HWND hWnd;
static char szClass[] = "Return to Ryll";
char error[256] = "Unspecified error";
BOOL sgfAppActive;
short leftMouseDown = 0, midMouseDown = 0, rightMouseDown = 0;
HCURSOR g_hCustomCursor;
HBITMAP hBit;

int single_game = 0, host_game = 0;

/* chat stuff */
int chat_mode = 0;
int inv_mode = 0;
char chat_buf[MAXCHATBUF];
int chat_buf_index = 0;

char dialog_bitmaps[MAXCOLORS][64] =
{
	 "data/Robots/SkinA.bmp",
	 "data/Robots/SkinB.bmp",
	 "data/Robots/SkinC.bmp",
	 "data/Robots/SkinD.bmp",
};

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	BOOL fRunning = TRUE;
	MSG	tMsg;
	WNDCLASS wc;
	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	int w, h;

	/* set up and register window class */
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = (WNDPROC) WndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = sizeof(DWORD);
	wc.hInstance = hInstance;
	wc.hIcon = NULL;

	wc.hCursor = g_hCustomCursor = LoadCursor(hInstance, MAKEINTRESOURCE(IDC_CURSOR1));
	wc.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = szClass;

	if (!RegisterClass(&wc))
	{
		MessageBox(NULL, "RegisterClass() failed!", "Error!", MB_OK);
		return FALSE;
	}

	w = GetSystemMetrics(SM_CXSCREEN) / 2;
	h = GetSystemMetrics(SM_CYSCREEN) / 2;

	/* create a window and display it */
	hWnd = CreateWindow(szClass, "Return to Ryll", WS_VISIBLE | WS_POPUP, 20, 20,
		400, 300, HWND_DESKTOP, NULL, hInstance, NULL);

	if (!hWnd)
	{
		MessageBox(NULL, "CreateWindow() failed!", "Error!", MB_OK);
		return FALSE;
	}

	hBit = (HBITMAP) LoadImage(NULL, "data/title.bmp",
		IMAGE_BITMAP, 0, 0, LR_DEFAULTSIZE | LR_LOADFROMFILE);

	GDIBlit(hWnd, hBit);

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	/* show dialog for IP and character color */
	DialogBox(hInstance, "IDD_DMZBEGIN", hWnd, (DLGPROC) DialogProc);

	srand(DMZ_RANDOM_SEED);

	dmz_map_init();
	dmz_object_init();

	if (single_game || host_game)
	{
		int rv;

		memset(&si, 0, sizeof(si));
		si.cb = sizeof(si);
		si.dwFlags = STARTF_USESHOWWINDOW;
		si.wShowWindow = SW_SHOWNA;

		dmz_logstd("[client:WinMain] single-player or hosting game, spawning server...");

		strcpy(server_ip, "127.0.0.1");

		rv = CreateProcess("ryllserv.exe", NULL, NULL, NULL, TRUE, 0, NULL, NULL, &si, &pi);

		if (rv == 0)
		{
			sprintf(buf, "[client:WinMain] server startup failed, error %d", GetLastError());
			dmz_logerr(buf);
		}
	}

	ShowCursor(FALSE);

	/* initialize the game */
	if (dmz_client_init(hWnd) != DMZ_SUCCESS)
	{
		dmz_client_kill();
		MessageBox(hWnd, error, "Error!", MB_OK | MB_ICONERROR);
		return FALSE;
	}

	ShowCursor(TRUE);

	/* run until game says stop */
	while (fRunning)
	{
		if (PeekMessage (&tMsg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (!GetMessage (&tMsg, NULL, 0, 0L))
				fRunning = FALSE;

			TranslateMessage (&tMsg);
			DispatchMessage (&tMsg);
		}

		else if (!sgfAppActive)
		{
			WaitMessage ();
		}

		else {
			if (!GameProcess ())
				DestroyWindow (hWnd);
		}
	}

	/* terminate the server */
	if (single_game)
		TerminateProcess(pi.hProcess, 0);

	return FALSE;
}

LRESULT CALLBACK WndProc(HWND hwnd, unsigned umsg, WPARAM wparam, LPARAM lparam)
{
	int i = 0;

	switch (umsg)
	{
	/* application is no longer active, don't run game! */
	case WM_ACTIVATEAPP:
		sgfAppActive = wparam;
		break;

	case WM_DESTROY:
		dmz_client_kill();
		PostQuitMessage(0);
		break;

	case WM_KEYDOWN:
		/* if we're in chat mode, echo keys pressed to screen */
		if (chat_mode == 1)
		{
			/* backspace removes one character */
			if (wparam == VK_BACK)
			{
				chat_buf_index--;
				
				/* if we're at the front - do nothing */
				if(chat_buf_index < 0)
					chat_buf_index = 0;

				/* terminate the string */
				chat_buf[chat_buf_index] = '\0';

				break;
			}

			/* only capture alphanumeric characters and spaces */
			else if ((wparam >= 0x30 && wparam <= 0x5A) || wparam == 0x20)
			{
				chat_buf[chat_buf_index] = wparam;
				chat_buf_index++;

				/* do not echo more than max */
				if (chat_buf_index >= MAXCHATBUF)
					chat_buf_index = MAXCHATBUF - 1;

				/* terminate the string */
				chat_buf[chat_buf_index] = '\0';
				break;
			}
		}

		/* not in chat mode, or non-alpha key */
		switch (wparam)
		{
		case VK_RETURN:

			/* if in chat mode, send message */
			if(chat_mode == 1)
			{
//				if (!local_game)
					do_chat(chat_buf);
//				else
//					LL_add(chatlist, chat_buf, time(NULL));
				
				/* get us out of chat mode */
				chat_mode = 0;

				/* reset chat buffer for next time we chat */
				for(i = 0; i < MAXCHATBUF; i++)
					chat_buf[i] = '\0';

				chat_buf_index = 0;
			}
			
			/* start chat mode if possible */
			else if (!inv_mode)
			{
				/* clear chat buffer */
				for(i = 0; i < MAXCHATBUF; i++)
					chat_buf[i] = '\0';

				chat_buf_index = 0;
				
				/* go to chat mode */
				chat_mode = 1;
			}

			break;

		case VK_SPACE: /* action */
			if (dmz_objects[my_id]->status == DEAD)
				dmz_client_restart();
			break;

		case 'I':		/* show inventory */
			if (!chat_mode && !inv_mode) inv_mode = 1;
			else inv_mode = 0;
			break;

		case 'N':		/* toggle player labels */
			show_names = !show_names;
			break;

		case VK_F1:		/* toggle help screen */
			show_help = !show_help;
			break;

		case VK_F11:	/* toggle frag screen */
			show_scores = !show_scores;
			break;

		case VK_F12:
			dmz_graph_screenshot("screenshot.bmp");
			break;

		case VK_ESCAPE: /* quit */
			show_exit = !show_exit;
			//DestroyWindow(hwnd);
			break;

		default:
			break;
		}
		break;

	case WM_LBUTTONDOWN:
		leftMouseDown = 1;
		break;

	case WM_MBUTTONDOWN:
		midMouseDown = 1;
		break;

	case WM_RBUTTONDOWN:
		rightMouseDown = 1;
		break;

	case WM_LBUTTONUP:
		leftMouseDown = 0;
		break;

	case WM_MBUTTONUP:
		midMouseDown = 0;
		break;

	case WM_RBUTTONUP:
		rightMouseDown = 0;
		break;

	default:
		return DefWindowProc(hwnd, umsg, wparam, lparam);
	}

	return 0L;
}


BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	int i = 0;
	static int bmtIndex = 0;
	static HBITMAP hBit[MAXCOLORS];
	static char localIP[16];
	static char ipaddr[16];
	HWND hDlgItem;
	HWND hFrame = GetDlgItem(hwndDlg, IDC_SKIN);
	
	switch (uMsg)
	{
		case WM_INITDIALOG:
		{
			char player[NAMELEN] = "";
			
			for (i = 0; i < MAXCOLORS; i++)
			{
				hBit[i] = (HBITMAP) LoadImage(NULL, dialog_bitmaps[i],
					IMAGE_BITMAP, 0, 0, LR_DEFAULTSIZE | LR_LOADFROMFILE);
			}

			/* default to single-player game */
			SendDlgItemMessage(hwndDlg, IDC_SINGLE, BM_SETCHECK, BST_CHECKED, 0);
			hDlgItem = GetDlgItem(hwndDlg, IDC_GROUP_MULTI);
			EnableWindow(hDlgItem, 0);
			hDlgItem = GetDlgItem(hwndDlg, IDC_HOST);
			EnableWindow(hDlgItem, 0);
			hDlgItem = GetDlgItem(hwndDlg, IDC_EDIT_IP);
			EnableWindow(hDlgItem, 0);
			hDlgItem = GetDlgItem(hwndDlg, IDC_TEXT_IP);
			EnableWindow(hDlgItem, 0);

			strcpy(ipaddr, DMZ_SERVER);

			dmz_get_config("DMZ", "Server Address", ipaddr, 16, DMZ_CONFIG_STRING);
			dmz_get_config("DMZ", "Last Player", player, NAMELEN, DMZ_CONFIG_STRING);

			SendDlgItemMessage(hwndDlg, IDC_EDIT_IP, WM_SETTEXT, 0, (LPARAM) ipaddr);
			SendDlgItemMessage(hwndDlg, IDC_EDIT_NAME, WM_SETTEXT, 0, (LPARAM) player);

			/* local IP retrieval */
			dmz_net_init();
			dmz_net_get_local_IP(localIP);
		}
		break;

		case WM_COMMAND:
		{
			switch (LOWORD(wParam))
			{
				case IDOK:
				{
					int rv;

					/* get server's IP address */
					SendDlgItemMessage(hwndDlg, IDC_EDIT_IP, WM_GETTEXT, 16, (LPARAM) server_ip);

					/* get player name */
					SendDlgItemMessage(hwndDlg, IDC_EDIT_NAME, WM_GETTEXT, 16, (LPARAM) player_name);

					/* check game type */
					rv = SendDlgItemMessage(hwndDlg, IDC_SINGLE, BM_GETCHECK, 0, 0);
					if (rv == BST_CHECKED) single_game = 1;

					/* check for debug mode */
					rv = SendDlgItemMessage(hwndDlg, IDC_HOST, BM_GETCHECK, 0, 0);
					if (rv == BST_CHECKED) host_game = 1;

					/* check if creating a new player */

					my_bitmap = bmtIndex + 1;

					EndDialog(hwndDlg, 0);
					return 1;
				}
				break;

				case IDCANCEL:
				{
					EndDialog(hwndDlg, 0);
					exit(1);
				}
				break;

				case IDC_BTN_NEXT:
				{
					bmtIndex++;
					if (bmtIndex >= MAXCOLORS) bmtIndex = 0;
				}
				break;

				case IDC_HOST:
				{
					int rv = SendDlgItemMessage(hwndDlg, IDC_HOST, BM_GETCHECK, 0, 0);

					if (rv == BST_CHECKED)
					{
						hDlgItem = GetDlgItem(hwndDlg, IDC_EDIT_IP);
						/* make server IP box read-only */
						SendDlgItemMessage(hwndDlg, IDC_EDIT_IP, EM_SETREADONLY, TRUE, 0);
						SendDlgItemMessage(hwndDlg, IDC_EDIT_IP, WM_SETTEXT, 0, (LPARAM) localIP);
					}

					else
					{
						hDlgItem = GetDlgItem(hwndDlg, IDC_EDIT_IP);
						/* make server IP box modifiable */
						SendDlgItemMessage(hwndDlg, IDC_EDIT_IP, EM_SETREADONLY, FALSE, 0);
						SendDlgItemMessage(hwndDlg, IDC_EDIT_IP, WM_SETTEXT, 0, (LPARAM) ipaddr);
					}
				}
				break;

				case IDC_SINGLE:
					hDlgItem = GetDlgItem(hwndDlg, IDC_GROUP_MULTI);
					EnableWindow(hDlgItem, 0);
					hDlgItem = GetDlgItem(hwndDlg, IDC_HOST);
					EnableWindow(hDlgItem, 0);
					hDlgItem = GetDlgItem(hwndDlg, IDC_EDIT_IP);
					EnableWindow(hDlgItem, 0);
					hDlgItem = GetDlgItem(hwndDlg, IDC_TEXT_IP);
					EnableWindow(hDlgItem, 0);

					break;

				case IDC_MULTI:
					hDlgItem = GetDlgItem(hwndDlg, IDC_GROUP_MULTI);
					EnableWindow(hDlgItem, 1);
					hDlgItem = GetDlgItem(hwndDlg, IDC_HOST);
					EnableWindow(hDlgItem, 1);
					hDlgItem = GetDlgItem(hwndDlg, IDC_EDIT_IP);
					EnableWindow(hDlgItem, 1);
					hDlgItem = GetDlgItem(hwndDlg, IDC_TEXT_IP);
					EnableWindow(hDlgItem, 1);

					break;
			}
		}
		break;
	}


	GDIBlit(hFrame, hBit[bmtIndex]);

	return 0;
}

void GDIBlit(HWND hWnd, HBITMAP hBit)
{
	HBITMAP hold;
	HDC hdc, memdc;
	RECT rect;
	BITMAP b;

	GetObject(hBit, sizeof(BITMAP), &b);
	GetClientRect(hWnd, &rect);

	hdc = GetDC(hWnd);
	memdc = CreateCompatibleDC(hdc);
	hold = SelectObject(memdc, hBit);
	
	StretchBlt(hdc, rect.left, rect.top,
		(rect.right - rect.left), (rect.bottom - rect.top),
		memdc, 0, 0, b.bmWidth, b.bmHeight, SRCCOPY);
	
	SelectObject(memdc, hold);
	ReleaseDC(hWnd, hdc);
	DeleteDC(memdc);
}
