/*	File:	attack.c
	Author:	Alex Fedosov, Jeff Malch
	Date:	March 10, 2001
	Description:	attack code
*/

#include <stdio.h>
#include <string.h>

#include "server.h"

#include "dmzcore.h"
#include "dmz_net.h"

char buf[256];

GLOBAL WORD16 bootybit;

/* perform hand-to-hand attack */
void HTHAttack(WORD16 attacker, WORD16 direction)
{
	WORD32 old_x = dmz_objects[attacker]->x;
	WORD32 old_y = dmz_objects[attacker]->y;
	int new_x = 0;
	int new_y = 0;
	WORD16 hthTarget;

	/* determine proper cell for hth attack based on direction */
	dmz_scale_delta(direction, &new_x, &new_y);
	new_x *= DEFAULT_VELOCITY;
	new_y *= DEFAULT_VELOCITY;

	dmz_objects[attacker]->x += new_x;
	dmz_objects[attacker]->y += new_y;

	hthTarget = (WORD16) DetectCollision(attacker);

	if(hthTarget != 0)
	{
		struct dmz_message m;
		WORD16 value = ApplyDamage(hthTarget, dmz_objects[attacker]->u.player.hth_dam);

		memset(&m, 0, sizeof(struct dmz_message));
		
		/* update of the attacker's score */
		m.objid = attacker;
		m.type = UPDATE;
		m.u.update.life = -1;
		m.u.update.status = -1;
		m.u.update.ammo = -1;
		m.u.update.frags = -1;

		/* update player's score */
		dmz_objects[attacker]->score += value;

		if (value != 0)	/* somebody died! */
		{
			/* send a death message and delete object */
			sprintf(buf, "[server:attack:HTHAttack] [%d] %s got killed", hthTarget, dmz_objects[hthTarget]->name);
			dmz_logstd(buf);

			/* increment player's frag count */
			dmz_objects[attacker]->u.player.frags++;

			/* send an update of the player */
			m.u.update.frags = dmz_objects[attacker]->u.player.frags;
			m.u.update.score = dmz_objects[attacker]->score;
			dmz_broadcast(&m);

			/* check to see if player needs an upgrade */
			UpgradePlayer(attacker);
		}
	}

	dmz_objects[attacker]->x = old_x;
	dmz_objects[attacker]->y = old_y;
}

/* apply damage to a target TODO update messages from here */
int ApplyDamage(WORD16 objid, short damage)
{
	WORD16 rv = 0;
	struct dmz_message m;
	memset(&m, 0, sizeof(struct dmz_message));

	/* update of the thing being killed */
	m.objid = objid;
	m.type = UPDATE;
	m.u.update.life = -1;
	m.u.update.score = -1;
	m.u.update.status = -1;
	m.u.update.ammo = -1;
	m.u.update.frags = -1;

	/* if already dead, nothing will happen */
	if (dmz_objects[objid]->status == DEAD)
		return rv;

	/* collision! somebody got hit */
	switch (dmz_objects[objid]->objtype)
	{
		/* decrement hitpoints appropriately */
		case PLAYER:
		case MONSTER:
			if (dmz_objects[objid]->u.player.life == 0)		/* unbreakable! */
				break;

			/* is it going to kill him? */
			if (dmz_objects[objid]->u.player.life > damage)
				dmz_objects[objid]->u.player.life -= damage;
			else
			{
				rv = dmz_objects[objid]->score;
				dmz_status_update(objid, DEAD);
				dmz_objects[objid]->u.player.life = 0;
				// TODO what to do when player dies?
				dmz_player_death(objid);
				dmz_objects[objid]->tstamp = time(NULL);
			}

			/* update the stats and broadcast */
			m.u.update.life = dmz_objects[objid]->u.player.life;
			dmz_broadcast(&m);
			break;

		case TERRAIN:
			if (dmz_objects[objid]->u.terrain.ter_hp == 0)		/* unbreakable! */
				break;

			if (dmz_objects[objid]->u.terrain.ter_hp > damage)
				dmz_objects[objid]->u.terrain.ter_hp -= damage;
			else
			{
				rv = dmz_objects[objid]->score;
				dmz_status_update(objid, DEAD);
				dmz_objects[objid]->u.terrain.ter_hp = 0;
				// TODO what to do when terrain dies?
			}

			/* update the stats and broadcast */
			m.u.update.life = dmz_objects[objid]->u.terrain.ter_hp;
			dmz_broadcast(&m);

			break;
		default:
			break;
	}

	return rv;
}

void MoveMissile(WORD16 mid)
{
	HOBJECT missile = dmz_objects[mid];

	/* decrease TTL */
	missile->u.missile.range--;

	/* destroy expired missiles, explosion and all */
	if (missile->u.missile.range == 0)
	{
		dmz_delete_object(mid);
	}

	/* out of range */
	else if (missile->x <= 0 ||
			 missile->x >= map_width * PIXELS_PER_CELL ||
			 missile->y <= 0 ||
			 missile->y >= map_height * PIXELS_PER_CELL)
	{
		/* destroy missile */
		dmz_delete_object(mid);
	}

	else
	{
		WORD16 target = 0;
		struct dmz_message m;
		int dx = 0, dy = 0;

		memset(&m, 0, sizeof(struct dmz_message));

		/* update of the attacker's score */
		m.type = UPDATE;
		m.u.update.life = -1;
		m.u.update.score = -1;
		m.u.update.status = -1;
		m.u.update.ammo = -1;
		m.u.update.frags = -1;

		/* update location */
		dmz_scale_delta(missile->u.missile.direction, &dx, &dy);
		dx *= missile->velocity;
		dy *= missile->velocity;
		target = DetectCollision(mid);

		/* no collision */
		if (target == 0)
		{
			/* put it in the new location */
			missile->x += dx;
			missile->y += dy;
		}

		/* two missiles collided, destroy both */
		else if (dmz_objects[target]->objtype == MISSILE)
		{
			dmz_delete_object(mid);
			dmz_delete_object(target);
		}

		/* collided with something */
		else
		{
			int value;
			WORD16 player = missile->u.missile.owner;

			value = ApplyDamage(target, missile->u.missile.damage);

			/* update player's score */
			dmz_objects[player]->score += value;

			if (value != 0)	/* somebody died! */
			{
				/* send a death message and delete object */
				sprintf(buf, "[server:attack:MoveMissile] [%d] %s got killed", target, dmz_objects[target]->name);
				dmz_logstd(buf);

				/* increment player's frag count */
				dmz_objects[player]->u.player.frags++;

				/* send an update of the player */
				m.objid = player;
				m.u.update.frags = dmz_objects[player]->u.player.frags;
				m.u.update.score = dmz_objects[player]->score;
				dmz_broadcast(&m);

				/* check to see if player needs an upgrade */
				UpgradePlayer(player);
			}

			/* delete the missile */
			dmz_delete_object(mid);
		}
	}
}

void dmz_player_death(WORD16 objid)
{
	WORD16 ammo_left = 10, life = 10;
	WORD16 booty = 0;
	struct dmz_message m_obj;

	/* get the ammo that is left over */
	if(dmz_objects[objid]->u.player.ammo > 0)
		ammo_left += dmz_objects[objid]->u.player.ammo;

	/* if the player is more experienced drop more ammo booty */
	ammo_left += (20 * (dmz_objects[objid]->u.player.level + 1));
	life += (20 * (dmz_objects[objid]->u.player.level + 1));

	if (ammo_left > DEFAULT_AMMO) ammo_left = DEFAULT_AMMO;
	if (life > DEFAULT_LIFE) life = DEFAULT_LIFE;

	booty = dmz_new_object(0, ITEM, (dmz_objects[objid]->x + 5), dmz_objects[objid]->y);
	dmz_objects[booty]->u.item.item_arm = life;
	dmz_objects[booty]->u.item.item_ammo = ammo_left;
	dmz_objects[booty]->u.item.item_frame = 0;
	dmz_objects[booty]->u.item.kind = BOOTY;
	dmz_objects[booty]->bitidx = bootybit;
	dmz_objects[booty]->width = DEFAULT_WIDTH;
	dmz_objects[booty]->height = DEFAULT_HEIGHT;

	memset(&m_obj, 0, sizeof(struct dmz_message));
	m_obj.type = OBJECT;
	m_obj.objid = dmz_objects[booty]->objid;
	memcpy(&(m_obj.u.object.obj), dmz_objects[booty], sizeof(struct dmz_object));

	sprintf(buf, "sending object %d -- %s bitmap %d", booty, dmz_objects[booty]->name, dmz_objects[booty]->bitidx);
	dmz_logstd(buf);

	dmz_broadcast(&m_obj);
}

